# Generated by cloudcoil-model-codegen v0.0.32
# DO NOT EDIT

from __future__ import annotations

from datetime import datetime
from typing import Annotated, Dict, List, Literal, Optional

from pydantic import Field

from cloudcoil import apimachinery
from cloudcoil.pydantic import BaseModel
from cloudcoil.resources import Resource


class EventSource(BaseModel):
    api_version: Annotated[
        Optional[str],
        Field(alias="apiVersion", description="API version of the referent"),
    ] = None
    kind: Annotated[
        Literal[
            "Bucket",
            "GitRepository",
            "Kustomization",
            "HelmRelease",
            "HelmChart",
            "HelmRepository",
            "ImageRepository",
            "ImagePolicy",
            "ImageUpdateAutomation",
            "OCIRepository",
        ],
        Field(description="Kind of the referent"),
    ]
    match_labels: Annotated[
        Optional[Dict[str, str]],
        Field(
            alias="matchLabels",
            description='MatchLabels is a map of {key,value} pairs. A single {key,value} in the matchLabels\nmap is equivalent to an element of matchExpressions, whose key field is "key", the\noperator is "In", and the values array contains only "value". The requirements are ANDed.',
        ),
    ] = None
    name: Annotated[str, Field(description="Name of the referent", max_length=53, min_length=1)]
    namespace: Annotated[
        Optional[str],
        Field(description="Namespace of the referent", max_length=53, min_length=1),
    ] = None


class ProviderRef(BaseModel):
    name: Annotated[str, Field(description="Name of the referent.")]


class AlertSpec(BaseModel):
    event_severity: Annotated[
        Optional[Literal["info", "error"]],
        Field(
            alias="eventSeverity",
            description="Filter events based on severity, defaults to ('info').\nIf set to 'info' no events will be filtered.",
        ),
    ] = "info"
    event_sources: Annotated[
        List[EventSource],
        Field(
            alias="eventSources",
            description="Filter events based on the involved objects.",
        ),
    ]
    exclusion_list: Annotated[
        Optional[List[str]],
        Field(
            alias="exclusionList",
            description="A list of Golang regular expressions to be used for excluding messages.",
        ),
    ] = None
    provider_ref: Annotated[
        ProviderRef,
        Field(alias="providerRef", description="Send events using this provider."),
    ]
    summary: Annotated[
        Optional[str],
        Field(description="Short description of the impact and affected cluster."),
    ] = None
    suspend: Annotated[
        Optional[bool],
        Field(
            description="This flag tells the controller to suspend subsequent events dispatching.\nDefaults to false."
        ),
    ] = None


class Condition(BaseModel):
    last_transition_time: Annotated[
        datetime,
        Field(
            alias="lastTransitionTime",
            description="lastTransitionTime is the last time the condition transitioned from one status to another.\nThis should be when the underlying condition changed.  If that is not known, then using the time when the API field changed is acceptable.",
        ),
    ]
    message: Annotated[
        str,
        Field(
            description="message is a human readable message indicating details about the transition.\nThis may be an empty string.",
            max_length=32768,
        ),
    ]
    observed_generation: Annotated[
        Optional[int],
        Field(
            alias="observedGeneration",
            description="observedGeneration represents the .metadata.generation that the condition was set based upon.\nFor instance, if .metadata.generation is currently 12, but the .status.conditions[x].observedGeneration is 9, the condition is out of date\nwith respect to the current state of the instance.",
            ge=0,
        ),
    ] = None
    reason: Annotated[
        str,
        Field(
            description="reason contains a programmatic identifier indicating the reason for the condition's last transition.\nProducers of specific condition types may define expected values and meanings for this field,\nand whether the values are considered a guaranteed API.\nThe value should be a CamelCase string.\nThis field may not be empty.",
            max_length=1024,
            min_length=1,
            pattern="^[A-Za-z]([A-Za-z0-9_,:]*[A-Za-z0-9_])?$",
        ),
    ]
    status: Annotated[
        Literal["True", "False", "Unknown"],
        Field(description="status of the condition, one of True, False, Unknown."),
    ]
    type: Annotated[
        str,
        Field(
            description="type of condition in CamelCase or in foo.example.com/CamelCase.",
            max_length=316,
            pattern="^([a-z0-9]([-a-z0-9]*[a-z0-9])?(\\.[a-z0-9]([-a-z0-9]*[a-z0-9])?)*/)?(([A-Za-z0-9][-A-Za-z0-9_.]*)?[A-Za-z0-9])$",
        ),
    ]


class AlertStatus(BaseModel):
    conditions: Optional[List[Condition]] = None
    observed_generation: Annotated[
        Optional[int],
        Field(
            alias="observedGeneration",
            description="ObservedGeneration is the last observed generation.",
        ),
    ] = None


class CertSecretRef(BaseModel):
    name: Annotated[str, Field(description="Name of the referent.")]


class SecretRef(BaseModel):
    name: Annotated[str, Field(description="Name of the referent.")]


class ProviderSpec(BaseModel):
    address: Annotated[
        Optional[str],
        Field(
            description="HTTP/S webhook address of this provider",
            pattern="^(http|https)://",
        ),
    ] = None
    cert_secret_ref: Annotated[
        Optional[CertSecretRef],
        Field(
            alias="certSecretRef",
            description="CertSecretRef can be given the name of a secret containing\na PEM-encoded CA certificate (`caFile`)",
        ),
    ] = None
    channel: Annotated[Optional[str], Field(description="Alert channel for this provider")] = None
    proxy: Annotated[
        Optional[str],
        Field(description="HTTP/S address of the proxy", pattern="^(http|https)://"),
    ] = None
    secret_ref: Annotated[
        Optional[SecretRef],
        Field(
            alias="secretRef",
            description='Secret reference containing the provider webhook URL\nusing "address" as data key',
        ),
    ] = None
    suspend: Annotated[
        Optional[bool],
        Field(
            description="This flag tells the controller to suspend subsequent events handling.\nDefaults to false."
        ),
    ] = None
    timeout: Annotated[
        Optional[str],
        Field(
            description="Timeout for sending alerts to the provider.",
            pattern="^([0-9]+(\\.[0-9]+)?(ms|s|m))+$",
        ),
    ] = None
    type: Annotated[
        Literal[
            "slack",
            "discord",
            "msteams",
            "rocket",
            "generic",
            "generic-hmac",
            "github",
            "gitlab",
            "bitbucket",
            "azuredevops",
            "googlechat",
            "webex",
            "sentry",
            "azureeventhub",
            "telegram",
            "lark",
            "matrix",
            "opsgenie",
            "alertmanager",
            "grafana",
            "githubdispatch",
        ],
        Field(description="Type of provider"),
    ]
    username: Annotated[Optional[str], Field(description="Bot username for this provider")] = None


class ProviderStatus(BaseModel):
    conditions: Optional[List[Condition]] = None
    observed_generation: Annotated[
        Optional[int],
        Field(
            alias="observedGeneration",
            description="ObservedGeneration is the last reconciled generation.",
        ),
    ] = None


class ResourceModel(BaseModel):
    api_version: Annotated[
        Optional[str],
        Field(alias="apiVersion", description="API version of the referent"),
    ] = None
    kind: Annotated[
        Literal[
            "Bucket",
            "GitRepository",
            "Kustomization",
            "HelmRelease",
            "HelmChart",
            "HelmRepository",
            "ImageRepository",
            "ImagePolicy",
            "ImageUpdateAutomation",
            "OCIRepository",
        ],
        Field(description="Kind of the referent"),
    ]
    match_labels: Annotated[
        Optional[Dict[str, str]],
        Field(
            alias="matchLabels",
            description='MatchLabels is a map of {key,value} pairs. A single {key,value} in the matchLabels\nmap is equivalent to an element of matchExpressions, whose key field is "key", the\noperator is "In", and the values array contains only "value". The requirements are ANDed.',
        ),
    ] = None
    name: Annotated[str, Field(description="Name of the referent", max_length=53, min_length=1)]
    namespace: Annotated[
        Optional[str],
        Field(description="Namespace of the referent", max_length=53, min_length=1),
    ] = None


class ReceiverSpec(BaseModel):
    events: Annotated[
        Optional[List[str]],
        Field(
            description="A list of events to handle,\ne.g. 'push' for GitHub or 'Push Hook' for GitLab."
        ),
    ] = None
    resources: Annotated[
        List[ResourceModel],
        Field(description="A list of resources to be notified about changes."),
    ]
    secret_ref: Annotated[
        SecretRef,
        Field(
            alias="secretRef",
            description="Secret reference containing the token used\nto validate the payload authenticity",
        ),
    ]
    suspend: Annotated[
        Optional[bool],
        Field(
            description="This flag tells the controller to suspend subsequent events handling.\nDefaults to false."
        ),
    ] = None
    type: Annotated[
        Literal[
            "generic",
            "generic-hmac",
            "github",
            "gitlab",
            "bitbucket",
            "harbor",
            "dockerhub",
            "quay",
            "gcr",
            "nexus",
            "acr",
        ],
        Field(
            description="Type of webhook sender, used to determine\nthe validation procedure and payload deserialization."
        ),
    ]


class ReceiverStatus(BaseModel):
    conditions: Optional[List[Condition]] = None
    observed_generation: Annotated[
        Optional[int],
        Field(
            alias="observedGeneration",
            description="ObservedGeneration is the last observed generation.",
        ),
    ] = None
    url: Annotated[
        Optional[str],
        Field(
            description="Generated webhook URL in the format\nof '/hook/sha256sum(token+name+namespace)'."
        ),
    ] = None


class Alert(Resource):
    api_version: Annotated[
        Optional[Literal["notification.toolkit.fluxcd.io/v1beta1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object.\nServers should convert recognized schemas to the latest internal value, and\nmay reject unrecognized values.\nMore info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "notification.toolkit.fluxcd.io/v1beta1"
    kind: Annotated[
        Optional[Literal["Alert"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents.\nServers may infer this from the endpoint the client submits requests to.\nCannot be updated.\nIn CamelCase.\nMore info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "Alert"
    metadata: Optional[apimachinery.ObjectMeta] = None
    spec: Optional[AlertSpec] = None
    status: Optional[AlertStatus] = None


class Provider(Resource):
    api_version: Annotated[
        Optional[Literal["notification.toolkit.fluxcd.io/v1beta1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object.\nServers should convert recognized schemas to the latest internal value, and\nmay reject unrecognized values.\nMore info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "notification.toolkit.fluxcd.io/v1beta1"
    kind: Annotated[
        Optional[Literal["Provider"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents.\nServers may infer this from the endpoint the client submits requests to.\nCannot be updated.\nIn CamelCase.\nMore info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "Provider"
    metadata: Optional[apimachinery.ObjectMeta] = None
    spec: Optional[ProviderSpec] = None
    status: Optional[ProviderStatus] = None


class Receiver(Resource):
    api_version: Annotated[
        Optional[Literal["notification.toolkit.fluxcd.io/v1beta1"]],
        Field(
            alias="apiVersion",
            description="APIVersion defines the versioned schema of this representation of an object.\nServers should convert recognized schemas to the latest internal value, and\nmay reject unrecognized values.\nMore info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
        ),
    ] = "notification.toolkit.fluxcd.io/v1beta1"
    kind: Annotated[
        Optional[Literal["Receiver"]],
        Field(
            description="Kind is a string value representing the REST resource this object represents.\nServers may infer this from the endpoint the client submits requests to.\nCannot be updated.\nIn CamelCase.\nMore info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"
        ),
    ] = "Receiver"
    metadata: Optional[apimachinery.ObjectMeta] = None
    spec: Optional[ReceiverSpec] = None
    status: Optional[ReceiverStatus] = None
